<?php
if ( ! class_exists( 'IPBar_Factory' )) {

    /**
     * Class for create share
     */
    class IPBar_Factory
    {
        /**
         * Bar Object
         *
         * @var object
         */
        protected $_bar;
        
        /**
         * Bar Object setting
         *
         * @var object
         */
        protected $_setting;
         
        /**
         * Class construct
         *
         * @return void
         */
        public function __construct( $setting = false )
        {
            if ( ! $setting ) 
            {
                $barId = 0;
                if ( $bar = $this->_get_bar_object() ) {
                    $barId = $bar->ID;
                }
                $this->bar         = $bar;
                $this->setting     = new IPBar_Setting( $barId, '__ipbar-setting' );
                $this->layout      = $this->setting->getSetting( 'layout' );
                $this->is_show     = $this->_is_show();
                $this->is_preview  = false;
            } else 
            {
                $bar = false;
                if ( ! empty( $_POST['barid'] )) 
                {
                    $barid = absint( $_POST['barid'] );
                    $bar   = get_post( $barid );
                }
                $this->bar         = $bar;
                $this->setting     = $setting;
                $this->is_show     = true;
                $this->is_preview  = true;
                $this->layout      = $this->setting->getSetting( 'layout' );
            }
        }
        
        /**
         * Get setting
         *
         * @return object
         */
        public function setting()
        {
            return $this->setting;
        }
        
        /**
         * Is show
         *
         * @return object
         */
        public function is_show()
        {
            return $this->is_show;
        }
        
        /**
         * Render bar
         *
         * @return void
         */
        public function render()
        {
            if ( ! $this->is_show() ) {
                return false;
            }
            $this->_get_script();
            $this->_get_bar();
        }
        
        /**
         * Get admin view file
         *
         * @param string $filename
         * @return void
         */
        public function view( $filename )
        {
            $path  = IPBAR_PATH . 'app/views/public/';
            $path .= strtolower( $filename ) . '.php'; 
            if ( file_exists( $path )) {
                include( $path );
            }
        }
        
        /**
         * Fix youtube url
         *
         * @param  string $url
         * @return string
         */
        public function fix_youtube_url( $url )
        {
            if ( $url )
            {
                if ( strpos( $url, 'embed' ) === false ) {
                    if ( strpos( $url, 'watch' ) !== false ) {
                        $id     = '';
                        $parsed = parse_url( $url );
                        if ( ! empty( $parsed['query'] )) {
                            parse_str( $parsed['query'], $args );
                            if ( ! empty( $args['v'] )) {
                                $id = $args['v'];
                            }
                        }
                        $url = 'http://youtube.com/embed/' . $id;
                    
                    } elseif ( strpos( $url, 'youtu.be' ) !== false ) {
                        $id     = '';
                        $parsed = parse_url( $url );
                        if ( ! empty( $parsed['path'] )) {
                            $id = trim( $parsed['path'], '/' );
                        }
                        $url = 'http://youtube.com/embed/' . $id;
                    
                    }
                }
                $url .= '?rel=0';
            }
            return $url;
        }
        
        /**
         * Get bar content
         *
         * @return void
         */
        protected function _get_bar()
        {
            $layout = $this->layout;
            $this->view( $layout );
        }
        
        /**
         * Get script
         *
         * @return void
         */
        protected function _get_script()
        {
            //
            //Get google font
            //
            $layout   = $this->layout;
            $columns  = $this->_get_columns( $layout );
            $families = array( 'roboto' );
            
            foreach( $columns as $column )
            {
                $key = 'column_' . $column . '_';
                $settings = array();
                $osetting = $this->setting->getSetting( 'column_' . $column );
                if ( ! empty( $osetting['settings'] )) 
                {
                    $type = 'text';
                    if ( ! empty( $osetting['type'] )) {
                        $type = $osetting['type'];
                    }
                    $setting = new IPBar_Setting_Data( $osetting['settings'] );
                    switch( $type )
                    {
                        //Text
                        case 'text' :
                            $families[] = $setting->getSetting( $key . 'text_font_family', 'roboto' );
                            break;
                        //Link
                        case 'link' :
                            $families[] = $setting->getSetting( $key . 'link_font_family', 'roboto' );
                            break;
                        //Facebook feeds
                        case 'facebook_feeds' :
                            $families[] = $setting->getSetting( $key . 'facebook_feeds_font_family', 'roboto' );
                            break;
                        //Twitter feeds
                        case 'twitter_feeds' :
                            $families[] = $setting->getSetting( $key . 'twitter_feeds_font_family', 'roboto' );
                            break;
                        //Recent Post
                        case 'recent_posts' :
                            $families[] = $setting->getSetting( $key . 'recent_posts_font_family', 'roboto' );
                            break;
                        //Countdown
                        case 'countdown' :
                            $families[] = $setting->getSetting( $key . 'countdown_font_family', 'roboto' );
                            break;
                        //Login form
                        case 'login_form' :
                            $families[] = $setting->getSetting( $key . 'login_form_font_family', 'roboto' );
                            break;
                        //Search form
                        case 'search_form' :
                            $families[] = $setting->getSetting( $key . 'search_form_font_family', 'roboto' );
                            break;
                    }
                }
            }
            $families = array_unique( $families );
            $fams = array();
            foreach( $families as $famname ) {
                $fams[] = $this->_get_google_font_family( $famname );
            }
            $family   = implode( '|', $fams );
            $fontUrl  = 'http://fonts.googleapis.com/css?family=' . $family;
            echo '<link id="ipbar-font" href="' . $fontUrl . '" rel="stylesheet" type="text/css">';
        }
        
        /**
         * Get google font family name
         *
         * @param  string $name
         * @return string|false
         */
        protected function _get_google_font_family( $name )
        {
            $family = false;
            $fonts  = IPBar_Font::getLinks();
            if ( array_key_exists( $name, $fonts )){
                $family = $fonts[$name];
            }
            return $family;
        }
        
        /**
         * Get bar object
         *
         * @return object
         */
        protected function _get_bar_object()
        {
            global $post;
            
            $barId   = 0;
            $bar     = false;
            $general = get_option( '__ipbar_general_bar' );
            if ( ! empty( $general )) {
                $barId = absint( $general );
            } else 
            {
                if ( ( is_page() || is_single() ) && ! empty( $post )) {
                    $barId = get_post_meta( $post->ID, '__ipbar_page_bar', true );
                } elseif ( is_home() ) {
                    $barId = get_option( '__ipbar_home_bar' );
                }
            }
            if ( ! empty( $barId )) {
                $bar = get_post( $barId );
            }
            return $bar;
        }
        
        /**
         * Check is bar object showing
         *
         * @return object
         */
        protected function _is_show()
        {
            if ( ! $this->bar || $this->bar->post_status != 'publish' ) {
                return false;
            }
            /* 
            $opt_key    = 'restricted_' . $this->_type;
            $restricted = $this->setting()->getSetting( $opt_key, 'false' );
            if ( $restricted == 'true' )
            {
                global $user_ID;
                if ( empty( $user_ID )) {
                    return false;
                }
            } 
            */
            return true;
        }
        
        /**
         * Get columns
         *
         * @return object
         */
        protected function _get_columns( $layout )
        {
            $columns = array(1);
            if ( strpos( $layout , 'two_column' ) !== false ) {
                $columns[] = 2;
            }
            elseif ( strpos( $layout , 'three_column' ) !== false ) {
                $columns[] = 2;
                $columns[] = 3;
            }
            return $columns;
        }
        
        //
        //Helpers
        //
        
        /**
         * Convert time to time elapsed string
         *
         * @param int|string $date
         * @return string
         */
        public function time_elapsed_string( $date )
        {
            $ptime = strtotime( $date );
            $etime = time() - $ptime;
            if ($etime < 1)
            {
                return 'Just now';
            }
            $a = array( 365 * 24 * 60 * 60  =>  'year',
                         30 * 24 * 60 * 60  =>  'month',
                              24 * 60 * 60  =>  'day',
                                   60 * 60  =>  'hour',
                                        60  =>  'minute',
                                         1  =>  'second'
                        );
            $a_plural = array( 'year'   => 'years',
                               'month'  => 'months',
                               'day'    => 'days',
                               'hour'   => 'hours',
                               'minute' => 'minutes',
                               'second' => 'seconds'
                        );

            foreach ($a as $secs => $str)
            {
                $d = $etime / $secs;
                if ($d >= 1)
                {
                    $r = round($d);
                    return $r . ' ' . ($r > 1 ? $a_plural[$str] : $str) . ' ago';
                }
            }
        }
    }
}